/*
Ankit Srivastava
MAT 259 - Visualizing Information - Winter 2012
Project 2 - Bivariate 2-D Visualization
*/

//Import libraries
import de.bezier.data.sql.*;

//Define Variables

int X = 1200;
int Y = 730;
PFont font1;
boolean showGrid = false;
boolean legend = true;
boolean showThreshold = false;
 
boolean[] showDewey;
int padding = 20;
int thresholdDays = 120;
int[] colors;
String[] deweyNames;
ArrayList transactions;

void setup(){
  size(X,Y);
  frameRate(30);
  colorMode(HSB,360,100,100,100);
  textAlign(CENTER, CENTER);
  smooth();
  initializeColors();
  initializeDewey();
  initializeBoolArray();
  loadData();
}


void draw(){
  background(40,0,100);
  if(!legend){
    pushMatrix();
    setOrigin();
    drawAxis();
    drawMarkers();
    plotData();
    popMatrix();
  }
  else {
    if(showGrid)
      drawLegendGrid();
    drawDeweyLegend();
  }
}

void initializeBoolArray(){
  showDewey = new boolean[10];
  for(int i=0;i<showDewey.length;i++){
    showDewey[i] = true;
  }
}

void loadData(){
  transactions = new ArrayList();
  String[] savedData = loadStrings("mainData.txt");
  for(int i=1; i<savedData.length;i++){
    String[] temp = split(savedData[i], "^|^");
    transactions.add( new Transaction(temp[0], temp[0], temp[2], temp[3], temp[4]) );    
  }
  
}

void plotData(){
  for(int i=0; i< transactions.size(); i++){
    
    Transaction t = (Transaction) transactions.get(i);
    int tcolor = floor(t.deweyClass/100);
    int choice = tcolor-1;
    if (choice < 0) 
      choice = 9;
    if(showDewey[choice]){
  
      if(showThreshold)
        if(t.duration < thresholdDays)
          continue;
  
      int talpha = floor(map(t.duration, 0, thresholdDays, 0, 100));
      //println(map(t.duration, 0, 100, 40, 100));
      Line l = new Line(0.5);
      //println(t.deweyClass+"  "+tcolor);
    
      l.setColor(colors[tcolor],80,80,talpha);
      l.setPoints(t.returnX(t.cout),t.returnY(t.cout),t.returnX(t.cin), t.returnY(t.cin));
      l.drawLine();
    }
  }
}
void initializeDewey(){
  deweyNames = new String[10];
  String savedData[] = loadStrings("deweyClasses.txt"); 
  for(int i=0; i<savedData.length; i++){
   String temp[] = split(savedData[i], "|");
   deweyNames[i] = temp[1] + " [" + temp[0] + "]";
  }
}

void initializeColors(){
  colors = new int[12];
  for(int i=0;i<10;i++)
    colors[i] = i*36;
}

void setOrigin(){
  
  translate(2*padding,Y-padding);
  //rotate(1.5*PI);
  fill(0,100,0,100);
  ellipse(0,0,5,5);
  
}

void drawAxis(){
  
  Line x = new Line(1.0);
  Line y = new Line(1.0);
  x.setColor(0,100,0,100);
  x.setPoints(0,0,X-4*padding,0);  
  x.drawLine();
  y.setColor(0,100,0,100);
  y.setPoints(0,0,0,Y-1.5*padding);  
  y.drawLine();
  
}

void drawMarkers(){
  
  for(int i=0;i<=24;i++){
    Line temp = new Line(0.5);
    temp.setColor(0,100,0,80);
    temp.setPoints(0,i*30, -10, i*30);
    temp.drawLine();
    
    textAlign(RIGHT,CENTER);
    font1 = loadFont("FreeSerif-12.vlw");
    textFont(font1);
    fill(0,100,0,100);
    if (i%2==0)  text(i, -12, -((i*30) +2.5 ));
    if(showGrid){
      Line grid = new Line(0.5);
      grid.setColor(0,100,0,60);
      grid.setPoints(0,i*30, 365*3, i*30);
      grid.drawLine();
    }
      
  }
  int yearText = 2009;
  int month = 1;
  int x = 0;
  for(int i=0; i<36; i++){
    
    Line temp = new Line(0.5);
    temp.setColor(0,100,0,80);
    if(month==2){
      x = x+28;
      temp.setPoints(x,0,x,-10);
    }
    else if((month==1) || (month==3) || month== 5 || month==7 || month==8 || month==10 || month==12){
      if (x==0)
        temp.setPoints(x,0,x,-10);  
      else{
        x = x+31;
        temp.setPoints(x,0,x,-10);  
      }
    }
    else {
      x = x+30;
      temp.setPoints(x,0,x,-10);
    }  
      
    temp.drawLine();
    
    if((i%12)==0){
      textAlign(CENTER,CENTER);
      font1 = loadFont("FreeSerif-12.vlw");
      fill(0,100,0,100);
      textFont(font1);
      text(yearText, x, 13);
      yearText += 1; 
    }
    if(month==12)
      month=1;
    else
      month++;  
    
    if(showGrid){
      Line grid = new Line(0.5);
      grid.setColor(0,100,0,60);
      grid.setPoints(x,0, x, 24*30);
      grid.drawLine();
    }  
  }
  
}

void drawLegendGrid() {
  for(int i=0;i < X; i=i+20){
    Line temp = new Line(0.5);
    if(i%10==0)
      temp.setColor(0,100,0,60);
    else
      temp.setColor(0,100,20,30);
    temp.setPoints(i,0, i, -Y);
    temp.drawLine();
  }
  
  for(int i=0;i < Y; i=i+20){
    Line temp = new Line(0.5);
    if(i%5==0)
      temp.setColor(0,100,0,60);
    else 
      temp.setColor(0,100,20,30);
    temp.setPoints(0,-i, X, -i);
    temp.drawLine();
  }
}
    
void drawDeweyLegend(){
  for(int i=0;i<10;i++){
    Line dewey = new Line(5);
    dewey.setPoints(X-7*padding,-(i*2*padding+160),X-4*padding,-(i*2*padding+160));
    dewey.setColor(colors[i], 80, 80, 100);
    dewey.drawLine();
    textAlign(RIGHT, CENTER);
    font1 = loadFont("FreeSerif-16.vlw");
    textFont(font1);
    fill(100,100,0,100);
    text(deweyNames[i].toUpperCase(), X-7*padding - padding/2, i*2*padding+158);
    textAlign(LEFT, CENTER);
    text("("+i+")", X-4*padding+padding/2, i*2*padding+158);
    
    //line(6*padding, 17*padding, 6*padding, 7*padding);
    //text(deweyNames[i], 1000+10*i, 300);
  }
  
  fill(0,100,20,60);
  textAlign(RIGHT,CENTER);
  text("Ankit Srivastava", X-2*padding, Y-2*padding);
  text("MAT259 - Winter 2012", X-2*padding, Y-1*padding);
  
  fill(100,100,0,100);
    ellipse(6*padding, 21*padding, 5,5);
    
    Line axis1 = new Line(1);
    axis1.setPoints(6*padding,-(22*padding),6*padding,-(12*padding));
    axis1.setColor(100, 80, 0, 100);
    axis1.drawLine();
    fill(0,100,0,100);
    font1 = loadFont("FreeSerif-16.vlw");
    textFont(font1);
    textAlign(LEFT, CENTER);
    text("TIME of DAY", 6.5*padding, 12*padding);
    
    Line axis2 = new Line(1);
    axis2.setPoints(5*padding,-21*padding,15*padding,-(21*padding));
    axis2.setColor(100, 80, 0, 100);
    axis2.drawLine();
    fill(100,100,0,100);
    font1 = loadFont("FreeSerif-16.vlw");
    textFont(font1);
    textAlign(LEFT, CENTER);
    text("DAY of YEAR", 15*padding, 21.5*padding);
    
    Line temp = new Line(1);
    temp.setPoints(8*padding,-17*padding,12*padding,-(14*padding));
    temp.setColor(0, 80, 80, 80);
    temp.drawLine();
    font1 = loadFont("FreeSerif-12.vlw");
    fill(100,100,0,80);
    textFont(font1);
    text("COUT", 8*padding+5, 17*padding+5);
    text("CIN", 12*padding+5, 14*padding+5);
    
    font1 = loadFont("FreeSerif-16.vlw");
    textFont(font1);
    fill(100,100,0,100);
    text("(G) - Toggle Grid", 6*padding, 27*padding);
    text("(L) - Toggle Legend", 6*padding, 29*padding);
    text("( ` ) - Toggle Dewey", 6*padding, 31*padding);
    text("(T) - Toggle Threshold", 6*padding, 33*padding);
    text("(1,2) - Toggle Dewey Category", 6*padding, 35*padding);
    
    textAlign(LEFT, CENTER);
    font1 = loadFont("FreeSerif-30.vlw");
    textFont(font1);
    fill(100,100,0,100);
    text("Bivariate Analysis of Least Popular Dewey Categories".toUpperCase(), 6*padding, 4*padding);
    
    
    
}

void keyPressed(){
  switch(key) {
    case 'G': showGrid = !showGrid;
              break;
    case 'g': showGrid = !showGrid;
              break;
    case 'L': legend = !legend;
              break;
    case 'l': legend = !legend;
              break;
    case '1': showDewey[0] = !showDewey[0];
              break;
    case '2': showDewey[1] = !showDewey[1];
              break;
    case '3': showDewey[2] = !showDewey[2];
              break;
    case '4': showDewey[3] = !showDewey[3];
              break;
    case '5': showDewey[4] = !showDewey[4];
              break;
    case '6': showDewey[5] = !showDewey[5];
              break;
    case '7': showDewey[6] = !showDewey[6];
              break;
    case '8': showDewey[7] = !showDewey[7];
              break;
    case '9': showDewey[8] = !showDewey[8];
              break;
    case '0': showDewey[9] = !showDewey[9];
              break;
    case 't': showThreshold = !showThreshold;
              break;
    case '`': for(int i=0;i<showDewey.length;i++)
                showDewey[i] = false;
              break;        
              
  }
  /*if (key=='G' || key=='g')
    showGrid = !showGrid;
    
  if (key=='L' || key=='l')
    legend = !legend;
  
  println(key);
  */
}


